<?php
namespace T3S\T3sHeaderslider\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Utility\GeneralUtility;


/**
 * HeadersliderController
 */
class HeadersliderController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * @var \TYPO3\CMS\Extbase\Service\FlexFormService
	 * @inject
	 */
	protected $flexFormService;

	/**
	 * @var \TYPO3\CMS\Core\Resource\FileRepository
	 * @inject
	 */
	protected $fileRepository;

	/**
	 * @var \TYPO3\CMS\Extbase\Service\ImageService
	 * @inject
	 */
	protected $imageService;

	/**
	 * @var \TYPO3\CMS\Frontend\Page\PageRepository
	 * @inject
	 */
	protected $pageRepository;

	/**
	 * @var array
	 */
	protected $fileObjects;


	/**
	* Initializes the current action
	*
	* @return void
	*/
	public function initializeAction() {

		$contentObjectData = $this->configurationManager->getContentObject()->data;
		$flexform = self::getFlexformData($contentObjectData['tx_t3sheaderslider']);
		$flexform = is_array($flexform) ? $flexform : [];
		$settings = array_merge($this->settings, $flexform);

		if ( $settings['slider'] ) {

			$currentSettings = $settings;

			// if recursive (default) ... media from rootline
			if ( $settings['recursive'] ) {
				$recursive = self::getRecursive($contentObjectData);
			} else {
				// if recursiveSettings (default) ... settings from rootline[0]
				if ($settings['recursiveSettings']) {
					$recursive = self::getFlexformData(self::getFrontendController()->rootLine[0]['tx_t3sheaderslider'], FALSE);
				}
			}

			if (is_array($recursive)) {
				$settings = array_merge($currentSettings, $recursive);
			}

			if ( $settings['sliderImages'] ) {

				if ( self::getFrontendController()->sys_language_content && $contentObjectData['_PAGES_OVERLAY_UID'] ) {
					#pages_overlay
					$uid = $settings['rootLineObjectData']['_PAGES_OVERLAY_UID'] ?
					(int)$settings['rootLineObjectData']['_PAGES_OVERLAY_UID'] : (int)$contentObjectData['_PAGES_OVERLAY_UID'];
					$this->fileObjects = $this->fileRepository->findByRelation('pages_language_overlay', 'headerslider', $uid);
				} else {
					#default language or no pages_overlay
					if ( self::getFrontendController()->sys_language_uid ) {
						if ($settings['rootLineObjectData']['_PAGES_OVERLAY_UID'] || $contentObjectData['_PAGES_OVERLAY_UID']) {
							$uid = $settings['rootLineObjectData']['_PAGES_OVERLAY_UID'] ?
							(int)$settings['rootLineObjectData']['_PAGES_OVERLAY_UID'] : (int)$contentObjectData['_PAGES_OVERLAY_UID'];
							$this->fileObjects = $this->fileRepository->findByRelation('pages_language_overlay', 'headerslider', $uid);
						} else {
							$uid = $settings['rootLineObjectData']['uid'] ?
							(int)$settings['rootLineObjectData']['uid'] : (int)$contentObjectData['uid'];
							$this->fileObjects = $this->fileRepository->findByRelation('pages', 'headerslider', $uid);
						}
					} else {
						#default language
						$uid = $settings['rootLineObjectData']['uid'] ?
						(int)$settings['rootLineObjectData']['uid'] : (int)$contentObjectData['uid'];
						$this->fileObjects = $this->fileRepository->findByRelation('pages', 'headerslider', $uid);
					}
				}
			}

			# EM-config languageMode
			if ( (empty($this->fileObjects) || $settings['recursive']) && $this->settings['languageMode'] && self::getFrontendController()->sys_language_uid ) {

				$page = self::getPageData($contentObjectData['uid']);
				$flexform = self::getFlexformData($page['tx_t3sheaderslider']);

				if (is_array($flexform)) {
					$settings = array_merge($currentSettings, $flexform);
					$recursive = self::getRecursive($contentObjectData);
					if (is_array($recursive)) {
						$settings = array_merge($currentSettings, $recursive);
					}
				}
				$uid = $settings['rootLineObjectData']['uid'] ?
				(int)$settings['rootLineObjectData']['uid'] : (int)$contentObjectData['uid'];
				$this->fileObjects = $this->fileRepository->findByRelation('pages', 'headerslider', $uid);
			}

			if ( (int)$currentSettings['recursiveSettings'] === 0 ) {
				$settings = $currentSettings;
			} else {
				if ( (int)$settings['recursiveSettings'] === 1 ) {
					$recursive = self::getFlexformData(self::getFrontendController()->rootLine[0]['tx_t3sheaderslider'], FALSE);
					if (is_array($recursive)) {
						$settings = array_merge($settings, $recursive);
					}
				}
			}

			if ($this->settings['plugin'] == 'nivoslider') {
				$notNull = ['width', 'height', 'pauseTime', 'animSpeed', 'slices', 'boxCols', 'boxRows'];
				foreach ( $notNull as $value ) {
					if (!$settings[$value]) {
						$settings[$value] = $this->settings[$value];
					}
				}
				if ( $settings['theme'] ) {
					$settings['themeWidth'] = $settings['width'];
					if ( $settings['theme'] == 'light' || $settings['theme'] == 'dark' ) {
						$settings['width'] -= 20;
					} elseif ( $settings['theme'] == 'bar' ) {
						$settings['width'] -= 2;
					}
				}
			} else {
				$notNull = ['width', 'height'];
				foreach ( $notNull as $value ) {
					if (!$settings[$value]) {
						$settings[$value] = $this->settings[$value];
					}
				}
			}

			$this->settings = $settings;

			$this->request->setControllerActionName($this->settings['plugin']);
		}
	}


	/**
	 * action for nivoslider
	 *
	 * @return void
	 */
	public function nivosliderAction()
	{
		$this->view->assign( 'fileObjects', $this->fileObjects );
	}


	/**
	 * Output a coustom slider view
	 *
	 * @return void
	 */
	public function customsliderAction()
	{
		$this->view->assign( 'fileObjects', $this->fileObjects );
	}


	/**
	 * Recursive data in settings array
	 * @var array $contentObjectData
	 *
	 * @return void
	 */
	public function getRecursive($contentObjectData)
	{
		$pagesOverlay = FALSE;
		if ( self::getFrontendController()->sys_language_content && $contentObjectData['_PAGES_OVERLAY_UID'] ) {
			$pagesOverlay = TRUE;
		}

		foreach ( self::getFrontendController()->rootLine as $rootLine ) {
			if ( $pagesOverlay ) {
				if ( $rootLine['tx_t3sheaderslider'] && ( $rootLine['uid'] != $contentObjectData['uid'] && $rootLine['_PAGES_OVERLAY_UID'] ) ) {
					$settings = self::getFlexformData($rootLine['tx_t3sheaderslider']);
					if ( $settings['sliderImages'] && !$settings['recursive'] ) {
						$settings['rootLineObjectData'] = $rootLine;
						return $settings;
					}
				}
			} else {
				if ( $rootLine['tx_t3sheaderslider'] && ( $rootLine['uid'] != $contentObjectData['uid'] ) ) {
					$settings = self::getFlexformData($rootLine['tx_t3sheaderslider']);
					if ( $settings['sliderImages'] && !$settings['recursive'] ) {
						$settings['rootLineObjectData'] = $rootLine;
						return $settings;
					}
				}
			}
		}
	}


	/**
	 * Returns the page data
	 *
	 * @return array
	 */
	protected function getPageData($pid, $sys_language_uid = 0)
	{
		$this->pageRepository->sys_language_uid = $sys_language_uid;

		return $this->pageRepository->getPage($pid);
	}


	/**
	 * Flexform data in settings array
	 * @var string $ffData: field tx_t3sheaderslider (flex) from pages
	 * @var boolean $images: from rootline if not on current page
	 *
	 * @return array
	 */
	public function getFlexformData($ffData, $images=TRUE)
	{
		$flexformArray = $this->flexFormService->convertFlexFormContentToArray($ffData);
		foreach ($flexformArray as $flexKey=>$flexValue) {
			if ( $images ) {
				$settings[$flexKey] = $flexValue;
			} else {
				if ($flexKey!='sliderImages') $settings[$flexKey] = $flexValue;
			}
		}
		return $settings;
	}


	/**
	 * Returns the frontend controller
	 *
	 * @return TypoScriptFrontendController
	 */
	protected function getFrontendController()
	{
		return $GLOBALS['TSFE'];
	}

}